## Copyright (C) 2012 Etienne Grossmann <etienne@egdn.net>
##
## This program is free software; you can redistribute it and/or modify it under
## the terms of the GNU General Public License as published by the Free Software
## Foundation; either version 3 of the License, or (at your option) any later
## version.
##
## This program is distributed in the hope that it will be useful, but WITHOUT
## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
## FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
## details.
##
## You should have received a copy of the GNU General Public License along with
## this program; if not, see <http://www.gnu.org/licenses/>.
##

## full_path = vrml_set_browser (proposed_browser) - Set or find a vrml browser
##
## If argument proposed_browser is passed and non-empty, then vmrl_set_browser()
## tries to determine whether proposed_browser is a valid program. If it is,
## then the global variable vrml_b_name is set to the full path, which is also
## returned. Otherwise, an error occurs.
##
## If argument proposed_browser is empty or missing, then vrml_set_browser()
## tries to determine whether either view3dscene, freewrl, whitedune or lookat
## (four known vrml browsers) are available. If one is found, then the global
## variable vrml_b_name is set to the full path, which is also returned.
## Otherwise, an error occurs.
##
## If proposed_browser is '-check', then the presence of all known browsers is
## checked and reported.
##
## WARNING: vrml_set_browser has only been tested under Linux. Please report
## successes / problems / patches on other systems.
##
## See also: vrml_browse(), FreeWRL homepage http://www.crc.ca/FreeWRL,
## whitedune homepage http://vrml.cip.ica.uni-stuttgart.de/dune.
##
function full_path = vrml_set_browser (proposed_browser_name)

global vrml_b_name = [];

full_path = vrml_b_name;

if nargin < 1
  proposed_browser_name = [];
end
if isempty (proposed_browser_name) || strcmp (proposed_browser_name, "-check")

  ## Should I check all packages, or just find one and exit?
  do_check = strcmp (proposed_browser_name, "-check");

  browser_list = {"view3dscene", "freewrl", "whitedune", "lookat"};
  available = zeros (1, numel (browser_list));
  full_paths = cell (1,  numel (browser_list));
  
  for i = 1:numel(browser_list)
    
    if do_check
      printf ("Checking for browser %15s on this system ...", browser_list{i});
    end
    ## TODO: Test this under windows
    [status, full_path] = system (sprintf ("which %s", browser_list{i}));
    if status != 1
      if full_path(length(full_path)) == "\n",
	    full_path = full_path(1:end-1);
      end
      available(i) = 1;
      full_paths{i} = full_path;
      if do_check
        printf (" Found it! :-)\n");
      else
        break;
      end
    else
      if do_check
        printf (" Could not find it :-(\n");
      end
    end
  endfor
  if ! any (available)
    printf ("No VRML browser is available.\n");

    ## Under Debian/Ubuntu, some of the above browsers are available, as of
    ## 2015 October 31. Point the user to apt-get / aptitude
    if regexp (uname().version, "(ubuntu|debian)", "ignorecase")
      printf (["\n\tYou may want to check whether your system has packages for some of the\n",...
               "\tabove-listed browsers.\n\n",...
               "\tTry 'apt-cache search <PACKAGE NAME>' from the command-line, or\n",...
               "\tsearch <PACKAGE NAME> in your system's package manager.\n\n"]);
    end
    if ! do_check
      error ("No VRML browser is available.");
    end
  else
    full_path = vrml_b_name = full_paths{find (available)(1)};
  end
  return
endif

argless_browser_name = proposed_browser_name(1,:);
args = "";
i_first_space = find (proposed_browser_name == ' ');
if ! isempty (i_first_space)
   argless_browser_name = proposed_browser_name(1:i_first_space(1)-1);
   args = proposed_browser_name(i_first_space(1)+1:end);
end

## TODO: Test this under windows
[status, full_path] = system (sprintf ("which %s", argless_browser_name));
if status == 1
  error (sprintf ("VRML browser `%s' is not available", argless_browser_name));
endif
if full_path(length(full_path)) == "\n",
  full_path = full_path(1:end-1);
end
full_path = [full_path, " ", args];
if rows (proposed_browser_name) > 1
  full_path = [full_path; proposed_browser_name(2:end,:)];
end
vrml_b_name = full_path
